function [f_of_X,K] = data_generator_clustering(n,p,d,m,I,seed,link)

% Creates the observations and the functions for a clustering multi-task
% algorithm, as described in the  simulations of the article. The functions
% in each cluster are all equal  (to f_1 or f_2).
%
% [f_of_X,K] = data_generator_clustering_article(n,p,d,alpha,z,m,I,seed,link)
%
% Input
% n : number of observations
% p : number of tasks
% d : dimension of the data space
% m : size of the linear combination of functions of the RKHS used to generate the f^j
% I : boolean vector indicating the two clusters. 
% seed : integer, used to initiate the pseudo-random generator.
% link : string, to describe the relations between the functions. size p*1
%   - 'ind' : f_1 and f_2 are drawn independantly
%   - 'opposite' : f_1 = -f_2
%
% Output
% f_of_X : matrix of f^j(X_i). Row : observations, columns : tasks. size n*p
% K : kernel matrix. size n*n


warning off;
rand('state',seed);
randn('state',seed);
warning on;

Ic = ones(p,1)-I;

% Data generation

X = randn(n,d);

alpha1 = randn(m,1);
alpha2 = randn(m,1);
z1 = randn(m,d);
z2 = randn(m,d);

% f_1
T1 = zeros(n,m);
for i = 1:n
    for j = 1:m
        T1(i,j) =  prod(exp(-abs(X(i,:)-z1(j,:))));
    end
end
f1 = sum(T1*alpha1,2);

% f_2
if strcmp(link,'ind'),
    T2 = zeros(n,m);
    for i = 1:n
        for j = 1:m
            T2(i,j) =  prod(exp(-abs(X(i,:)-z2(j,:))));
        end
    end
    f2 = sum(T2*alpha2,2);
elseif strcmp(link,'opposite'),
    f2 = -f1;
else f2 = NaN;
end

fX = kron(I,f1) + kron(Ic,f2);
f_of_X = compact(fX,n,p);

K = zeros(n,n);
for i = 1:n
    K(i,i) = 1;
    for j = i+1:n
        K(i,j) = prod(exp( -( abs( X(i,:)-X(j,:) ) ) ) );
        K(j,i) = K(i,j);
    end
end    
end