function s = single_task(f,df,g,method)

% Estimates the variance of the noise in a single-task problem, using
% minimal penalties.
%
% s = single_task(f,df,g,methode)
%
% Input :
% f : vector of the empirical errors ||Y_z-A_l*Y_z||/n. size L*1
% df : vector of the generalized degree of freedom. size 1*L
% g : vector of the minimal penalties. size L*1
% method : string which indicates how to estimate the variance.
%   - 'half' : takes the smallest index such that df < L/2
%   - 'largest_jump' : takes the largest jump of df.
%
% Output:
% s : estimator of the variance of the noise.

% Computes the grid of C. Implements Algorithm 2 of the article
% "Data-driven calibration of penalties for least squares regression" (S.
% Arlot & P. Massart, JMLR 2009)

C = 0;
l = find_min(f');
d = df(l);
f0 = f(l);
g0 = g(l);
G = (f>f0) & (g<g0); 
[~,L] = size(df);
n = L-1;

warning off;
while sum(G)>0
   T = ( abs((f-f0) ./ (g0-g)) ./ G);
   K = min(T);
   C = [C K];
   l = find_min(T');
   f0 = f(l);
   g0 = g(l);
   G = (f>f0) & (g<g0); 
   d = [d df(l)];
end
warning on;

nbC = size(C,2);

% Estimation of sigma
if strcmp(method,'largest_jump'),
    di = [d d(nbC)];
    di1 = [d(1) d];
    diff = di - di1;
    pos = trouver_min(diff);
    s = C(pos);
elseif strcmp(method,'half'),
       indices = d > n/2;
       pos = sum(indices)+1;
       s = C(pos);
else s = NaN;    
end    
end


    


